//
// Created by dell on 24-12-28.
//

#include "soft_i2c.h"

/*!
    \brief      delay
    \param[in]  none
    \param[out] none
    \retval     none
*/
void soft_i2c_delay(void) {
    delay_us(20); // Adjust delay as needed
    /* delay to freq
     *  20KHz: delay_us(20);
     *  65KHz: delay_us(1);
     */
}

/*!
    \brief      configure the software IIC GPIO
    \param[in]  none
    \param[out] none
    \retval     none
*/
void soft_i2c_config(void) {
    rcu_periph_clock_enable(RCU_GPIO_I2C);

    gpio_mode_set(I2C_SCL_PORT, GPIO_MODE_OUTPUT, GPIO_PUPD_PULLUP, I2C_SCL_PIN);
    gpio_output_options_set(I2C_SCL_PORT, GPIO_OTYPE_OD, GPIO_OSPEED_50MHZ, I2C_SCL_PIN);

    gpio_mode_set(I2C_SDA_PORT, GPIO_MODE_OUTPUT, GPIO_PUPD_PULLUP, I2C_SDA_PIN);
    gpio_output_options_set(I2C_SDA_PORT, GPIO_OTYPE_OD, GPIO_OSPEED_50MHZ, I2C_SDA_PIN);

    I2C_SCL_HIGH();
    I2C_SDA_HIGH();
}

/*!
    \brief      generate I2C start signal
    \param[in]  none
    \param[out] none
    \retval     none
*/
void soft_i2c_start(void) {
    // sda_out();
    I2C_SDA_HIGH();
    I2C_SCL_HIGH();
    soft_i2c_delay();
    I2C_SDA_LOW();
    soft_i2c_delay();
    I2C_SCL_LOW();
    // soft_i2c_delay();
    // 从全高开始，SCL为高期间，SDA下降沿表示start信号，再拉低SCL
}

/*!
    \brief      generate I2C stop signal
    \param[in]  none
    \param[out] none
    \retval     none
*/
void soft_i2c_stop(void) {
    // sda_out();
    I2C_SCL_LOW();
    I2C_SDA_LOW();
    soft_i2c_delay();
    I2C_SCL_HIGH();
    soft_i2c_delay();
    I2C_SDA_HIGH();
    // 从全低开始，SCL为高期间，SDA上升沿表示stop
}

/*!
    \brief      send I2C ACK signal
    \param[in]  none
    \param[out] none
    \retval     none
*/
void soft_i2c_send_ack(void) {
    // sda_out();
    I2C_SDA_LOW();
    soft_i2c_delay();
    I2C_SCL_HIGH();
    soft_i2c_delay();
    I2C_SCL_LOW();
    soft_i2c_delay();
    I2C_SDA_HIGH();
    // SCL产生一个正常的时钟周期，其间SDA始终为低电平，表示ACK
}

/*!
    \brief      send I2C NACK signal
    \param[in]  none
    \param[out] none
    \retval     none
*/
void soft_i2c_send_nack(void) {
    // sda_out();
    I2C_SDA_HIGH();
    soft_i2c_delay();
    I2C_SCL_HIGH();
    soft_i2c_delay();
    I2C_SCL_LOW();
    soft_i2c_delay();
    I2C_SDA_HIGH();
    // SCL产生一个正常的时钟周期，其间SDA始终为高电平，表示NACK
}

/*!
    \brief      wait I2C ACK signal
    \param[in]  none
    \param[out] none
    \retval     0: ACK received, 1: ACK not received
*/
uint8_t soft_i2c_wait_ack(void) {
    // sda_in();
    I2C_SDA_HIGH();
    soft_i2c_delay();
    I2C_SCL_HIGH();
    soft_i2c_delay();
    uint8_t ack = !I2C_SDA_READ();
    //ACK信号是第九个时钟期间，SDA被从机在SCL高期间，拉低并保持低电平。此处判断SDA是否被拉低，被拉低则返回0，取反为1，表示收到ACK
    I2C_SCL_LOW();
    return ack;
}

/*!
    \brief      send a byte via I2C
    \param[in]  byte: byte to be sent
    \param[out] none
    \retval     none
*/
void soft_i2c_send_byte(uint8_t byte) {
    // sda_out();
    for (int i = 0; i < 8; i++) {
        if (byte & 0x80) { //通过&操作获取第一位是1还是0
            I2C_SDA_HIGH(); //SCL低电平期间，SDA高电平表示1
        } else {
            I2C_SDA_LOW(); //SCL低电平期间，SDA低电平表示0
        }
        byte <<= 1; //左移一位，把原本第二位的数据移到第一位，再判断高低电平
        soft_i2c_delay();
        I2C_SCL_HIGH(); //SCL拉高电平，SDA电平状态保持不变
        soft_i2c_delay();
        I2C_SCL_LOW(); //SCL拉低电平
        delay_us(5);
    }
    // i2c_wait_ack();
}

/*!
    \brief      receive a byte via I2C
    \param[in]  ack: 1: send ACK, 0: send NACK
    \param[out] none
    \retval     received byte
*/
uint8_t soft_i2c_receive_byte(uint8_t ack) {
    // sda_in();
    uint8_t byte = 0;
    I2C_SDA_HIGH(); //从高开始
    for (int i = 0; i < 8; i++) {
        byte <<= 1;
        I2C_SCL_HIGH();
        soft_i2c_delay();
        if (I2C_SDA_READ()) {
            byte |= 0x01;
        }
        I2C_SCL_LOW();
        soft_i2c_delay();
    }
    if (ack) {
        soft_i2c_send_ack();
    } else {
        soft_i2c_send_nack();
    }
    return byte;
}

uint8_t soft_i2c_write_16bits(uint8_t slave_addr, uint8_t reg_addr, uint8_t data[2]) {
    soft_i2c_start();
    soft_i2c_send_byte(slave_addr);
    if (!soft_i2c_wait_ack()) {
        soft_i2c_stop();
        return SOFT_I2C_FAIL;
    }
    soft_i2c_send_byte(reg_addr);
    if (!soft_i2c_wait_ack()) {
        soft_i2c_stop();
        return SOFT_I2C_FAIL;
    }
    soft_i2c_send_byte(data[0]);
    if (!soft_i2c_wait_ack()) {
        soft_i2c_stop();
        return SOFT_I2C_FAIL;
    }
    soft_i2c_send_byte(data[1]);
    if (soft_i2c_wait_ack()){}
    soft_i2c_stop();
    return SOFT_I2C_OK;
}

uint8_t soft_i2c_read_16bits(uint8_t slave_addr, uint8_t reg_addr, uint8_t *data)
{
    soft_i2c_start();
    soft_i2c_send_byte(slave_addr);
    if (!soft_i2c_wait_ack()) {
        soft_i2c_stop();
        return SOFT_I2C_FAIL;
    }
    soft_i2c_send_byte(reg_addr);
    if (!soft_i2c_wait_ack()) {
        soft_i2c_stop();
        return SOFT_I2C_FAIL;
    }
    soft_i2c_start();
    soft_i2c_send_byte(slave_addr | 0x01);
    if (!soft_i2c_wait_ack()) {
        soft_i2c_stop();
        return SOFT_I2C_FAIL;
    }
    soft_i2c_delay();
    data[0] = soft_i2c_receive_byte(1);
    data[1] = soft_i2c_receive_byte(0);
    soft_i2c_stop();
    return SOFT_I2C_OK;
}